#-------------------------------------------------------------------------------
# Клиентское приложение для ректификации с "железом" варианта LITE
# Версия 0.0.1.4
# OldBean, 16.02.2019, 08.08.2019
#-------------------------------------------------------------------------------
import tkinter as tk # Библиотека для создания GUI
import redis # Для работы с СУБД redis
import matplotlib.pylab as plt # Библиотека для построения графиков (и ниже)
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
from matplotlib.figure import Figure
from matplotlib.gridspec import GridSpec
import numpy as np # Для быстрой работы с массивами
import time # Функции, связанные со временем
import random # Генераторы псевдослучайных чисел
import threading # Средства для организации многопоточности

""" Чтобы не нужно было вносить изменения в скрипт, необходимо выполнить
    следующие условия по "железу". Под словом "первый" понимается устройство,
    имеющее наименьший адрес на шине I2C следи устройств этого же семейства.
    1. Предполагается, что измерения температуры производится датчикими 
       DS18B20, которые подключены к каналам 0-4 первого 1-Wire хаба. Порядок
       подключения датчиков к каналам хаба следующий: 0 - датчик в кубе, 
       1 - датчик в нижней части колонны, 2 - датчик в дефлегматоре, 3 - датчик 
       на трубке ТСА, 4 - датчик воды охлаждения на выходе дефлегматора.
    2. Первое твердотельное реле - это реле контактора для включения/выключения
       высокого напряжения.
    2. Первый датчик RMS измеряет напряжение сети, из которой питаются ТЭНы
    3. Первый контроллер PDM регулирует мощность нагрева основного ТЭНа,
       расположенного в кубе.
    4. Первый контроллер PWM связан с клапаном отбора у дефлегматора
"""
#===============================================================================
# Основные параметры
#===============================================================================
lt = 1200 # Глубина истории отображаемая на графике в секундах
twcr = 30.0 # Критическая температура охлаждающей воды на выходе дефлегматора
ttcr = 50.0 # Критическая температура ТСА (трубки связи с атмосферой)
tcrcr = 65.0 # Температура в нижней части колонны для окончания режима разгона
tdcr = 78.5 # Температура дефлегматора для окончания отбора хвостов
tsetmin = 76.0 # Ограничитель уставки регулятора отбора снизу
tsetmax = 85.0 # Ограничитель уставки регулятора отбора сверху
crefr = 0 # Счетчик секунд (для обновления графика)
nrefr = 5 # Период обновления графиков температур в секундах
tset = 78.13 # Уставка регулятора
dt = 0.0625 # Квант датчика DS18B20
baseq = 0.0 # Базовая скорость отбора
curq = baseq # Текущаяя скорость отбора
error = False # Флаг ошибки
timest = time.time() # Начало отсчета таймера. Актуализируется кнопкой таймера
timetxt = '0:00:00:00'
#===============================================================================
# Анализ конфигурации (из БД) и "привязка к местности" :)
#===============================================================================
r = redis.StrictRedis() # Соединяемся с базой данных

ckey = None # Ключ первого твердотельного реле
rmskey = None # Ключ первого датчика напряжения
tkey = None # Ключ контроллера ТЭНа
vkey = None # Ключ основного клапана отбора
akey = None # Ключ датчика атмосферного давления (BMP)
# Получим из БД отсортированный список ключей хэш-таблиц подключенных устройств
keys = sorted(r.keys('ix*')) 
for key in keys: # Пробегаем по списку ключей
  fc = int(r.hget(key, b'fc')) # Код семейства устройства с данным ключем
  if fc == 0xA1 and rmskey is None: rmskey = key[1:4]
  elif fc == 0xA2 and ckey is None: ckey = key[1:4]
  elif fc == 0xA3 and tkey is None: tkey = key[1:4]
  elif fc == 0xA4 and vkey is None: vkey = key[1:4]
  elif fc == 0x77 and akey is None: akey = key[1:4]
dkeys = [rmskey, ckey, tkey, vkey, akey]
dnames = ['Сеть', 'Контактор', 'ТЭН', 'Отбор', 'Атмосфера']
print('Ключи исполнительных устройств и датчиков:')
for key in dkeys:
  print(key, dnames[dkeys.index(key)])
  
# Получим сортированный список ключей всех подключенных 1-Wire устройств
keys = sorted(r.keys('ix*_*'))
# Первые пять ключей как раз и будут ключами хэшей датчиков, подключенных к
# первым каналам первого 1-Wire хаба
tpkey = keys[0][1:6] # Ключ датчика температуры в кубе
tckey = keys[1][1:6] # Ключ датчика температуры в нижней части колонны
tdkey = keys[2][1:6] # Ключ датчика температуры в дефлегматоре
ttkey = keys[3][1:6] # Ключ датчика температуры в ТСА
twkey = keys[4][1:6] # Ключ датчика температуры воды на выходе из дефлегматора
ppkey = None # Ключ дифференциального датчика давления в кубе
for i in range(5, len(keys)): # Ищем датчик диф. давления в кубе
  fc = int(r.hget(keys[i], b'fc')) # Код семейства устройства с данным ключем
  if fc == 0xA7: # Нашли, таки...
    ppkey = keys[i][1:6]
    break

tskeys = [tpkey, tckey, tdkey, ttkey, twkey]
tsnames = ['Куб', 'Колонна', 'Дефлегм.', 'ТСА', 'Вода']
for key in tskeys:
  print(key, tsnames[tskeys.index(key)] + ' (темп-ра)')
print(ppkey, 'Диф. давление в кубе')

wmax = float(r.hget(b'i' + tkey, b'max'))
print('Макс. мощность нагрева ТЭНа - %.1f Вт'%(wmax))
qmax = float(r.hget(b'i' + vkey, b'max'))
print('Макс. скорость отбора - %.1f мл/час'%(qmax))

#===============================================================================
# Формирование и настройка GUI
#===============================================================================
root = tk.Tk() # Создаем главное окно приложения
frame = tk.Frame(width = 166) # Левая инструментальная панель
frame.pack(side = tk.LEFT, fill = tk.Y)
yb = 0 # Текущая y-координата виджета для place-компоновки
lw = 164 # Ширина разделительной линии
hwg = 20 # Высота однострочного виджета
cc = ['b', 'r', 'g', 'c', 'm', 'y', 'k'] # Цвета для графиков и полей
#===============================================================================
# Таймер
#===============================================================================
def timestr(dt): # Перевод временного интервала в формат Д:ЧЧ:ММ:СС
  t = int(dt) # Отбрасываем доли секунды
  d = t//86400; t -= d*86400 # Количество суток
  h = t//3600; t -= h*3600 # Количество часов
  m = t//60; t -= m*60 # Количество минут
  s = t
  timetxt = '%1d:%02d:%02d:%02d'%(d, h, m, s)
  return timetxt

def timer(): # Callback-функция для кнопки сброса таймера
  global timest, timetxt
  timest = time.time()

tk.Label(frame, text = 'Таймер', width = 6, anchor = 'w').place(x = 5, y = yb)
# Окно для отображения времени
timerlab = tk.Label(frame, width = 10, anchor = 'e', fg = 'dark green')
timerlab.place(x = 54, y = yb)
 
# Картинки для кнопки в кодировке base64
itimer =  '''R0lGODlhCQAJAIABAP8AAP///yH5BAEKAAEALAAA
             AAAJAAkAAAIPjI8Gm+vnooos0GoleqkXADs='''
imgtimer = tk.PhotoImage(data = itimer)
iempty = '''R0lGODlhCQAJAIABADOZM////yH5BAEKAAEALAA
              AAAAJAAkAAAIIjI+py+0PTwEAOw=='''
imgempty = tk.PhotoImage(data = iempty)

# Кнопка для сброса таймера
buttimer = tk.Button(frame, image = imgtimer, command = timer)
buttimer.place(x = 148, y = yb + 1)
citimer = imgtimer 
yb += hwg
#-------------------------------------------------------------------------------
tk.Frame(frame, width = lw, height = 1, bg = 'grey').place(x = 0, y = yb); yb += 2
#===============================================================================
# Контактор
#===============================================================================
vchb = tk.IntVar() # Целая переменная, ассоциированная с контактором
def fchb(): # Callback-функция чекбокса контактора
  r.lset(ckey, -1, vchb.get())
  r.rpush(b'must', ckey)
cchb = tk.Checkbutton(frame, text = ' ' + dnames[dkeys.index(ckey)],
                      command = fchb, variable = vchb, width = 17, anchor = 'w')
cchb.place(x = 0, y = yb)
yb += hwg
#-------------------------------------------------------------------------------
tk.Frame(frame, width = lw, height = 1, bg = 'grey').place(x = 0, y = yb); yb += 2
#===============================================================================
# Режимы работы
#===============================================================================
tk.Label(frame, text = 'Режимы работы:', width = 19, anchor = 'w').place(x = 5, y = yb)
yb += hwg

class MODE(object):
  """Класс режимов работы установки"""
  def __init__(self, name = '', next = 2, w = 0, q = 0, callback = None):
    self.name = name # Наименование режима
    self.next = next # Номер следующего режима (после завершения работы данного)
    self.w = w # Мощность нагрева ТЭНа
    self.q = q # Скорость отбора

mode = tk.IntVar(0) # Текущий режим работы (связан с радиокнопками)
modes = [] # Список доступных режимов работы

modes.append(MODE(name = 'Мониторинг', next = 0))
modes.append(MODE(name = 'Разгон', w = wmax, q = 0))
modes.append(MODE(name = 'Холостой ход', w = 600, q = 0))
modes.append(MODE(name = 'Головы', w = 600, q = 50))
modes.append(MODE(name = 'Подголовники', w = 600, q = 500))
modes.append(MODE(name = 'Тело', w = 600, q = 500))
modes.append(MODE(name = 'Хвосты', w = 600, q = 500))
modes.append(MODE(name = 'Пропарка колонны', w = wmax, q = qmax))
modes.append(MODE(name = 'Слив', w = 0, q = qmax))
modes.append(MODE(name = 'СР2', w = 600, q = 250))

def rbcom(): # Callback-функция, вызываемая при переключении режимов
  global baseq, curq
  cm = mode.get() # Номер нового режима
  w = modes[cm].w # Мощность нагрева для данного режима
  r.lset(tkey, -1, w); r.rpush(b'must', tkey) # Установим мощность нагрева
  curq = baseq = modes[cm].q # Базовая скорость отбора для данного режима
  r.lset(vkey, -1, curq); r.rpush(b'must', vkey) # Выключаем отбор

rbms = [] # Список радиокнопок для переключения режимов
for mi in range(len(modes)): # Определяем виджеты и устанавливаем их на панели
  rb = tk.Radiobutton(frame, text = modes[mi].name, variable = mode,
                      value = mi, command = rbcom, width = 17, anchor = 'w')
  rb.place(x = 0, y = yb)
  rbms.append(rb)
  yb += hwg
#-------------------------------------------------------------------------------
tk.Frame(frame, width = lw, height = 1, bg = 'grey').place(x = 0, y = yb); yb += 2
#===============================================================================
# Контроль температуры ТСА
#===============================================================================
ttcontr = tk.IntVar() # Целая переменная, ассоциированная с контролем т-ры ТСА
ttcontr.set(0)
def fttcontr(): # Callback-функция чекбокса вкл/выкл контроля т-ры ТСА
  pass
#  if ttcontr.get() == 0:
#    print('Контроля т-ры ТСА нет')
#  else:
#    print('Контроль т-ры ТСА есть')
ttchb = tk.Checkbutton(frame, text = ' Контроль т-ры ТСА', command = fttcontr,
                       variable = ttcontr, width = 17, anchor = 'w')
ttchb.place(x = 0, y = yb)
yb += hwg
#-------------------------------------------------------------------------------
tk.Frame(frame, width = lw, height = 1, bg = 'grey').place(x = 0, y = yb); yb += 2
#===============================================================================
# Температуры
#===============================================================================
tk.Label(frame, text = 'Температуры,°C:', width = 18, anchor = 'w').place(x = 5, y = yb)
yb += hwg

colors = ['blue', 'red', 'dark green', 'dark cyan', 'dark magenta', 'yellow',
          'black', 'orange']
Tlabs = []
for i in range(len(tskeys)):
  tk.Label(frame, text = tsnames[i], width = 9, anchor = 'w',
           fg = colors[i]).place(x = 5, y = yb)
  T = '%.3f'%(float(r.lindex(tskeys[i], -1)))
  lab = tk.Label(frame, text = T, width = 7, anchor = 'e', fg = colors[i])
  lab.place(x = 100, y = yb)
  Tlabs.append(lab)
  yb += hwg
#-------------------------------------------------------------------------------
tk.Frame(frame, width = lw, height = 1, bg = 'grey').place(x = 0, y = yb); yb += 2
#===============================================================================
# Уставка
#===============================================================================
tk.Label(frame, text = 'Уставка', width = 9, anchor = 'w').place(x = 5, y = yb)

settxt = tk.StringVar()
settxt.set('%.3f'%(tset))
def setplus(): # Callback-функция для кнопки '+'
  global tset
  tset += dt
  if tset > tsetmax:
    tset = tsetmax
  settxt.set('%.3f'%(tset))
def setminus(): # Callback-функция для кнопки '-'
  global tset
  tset -= dt
  if tset < tsetmin:
    tset = tsetmin
  settxt.set('%.3f'%(tset))
def setcheck(): # Callback-функция для кнопки 'v'
  global tset
  tset = float(r.lindex(tckey, -1))
  settxt.set('%.3f'%(tset))

# Картинки для кнопок в кодировке base64
iplus  = '''R0lGODlhCQAJAIABAP8AAP///yH5BAEKAAEALAA
            AAAAJAAkAAAIOjI8HqWyLgJywuZqwMwUAOw=='''
iminus = '''R0lGODlhCQAJAIABAAAA/////yH5BAEKAAEALAA
            AAAAJAAkAAAIKjI+pywEPQZt0FQA7'''
icheck =  '''R0lGODlhCQAJAIABADOZM////yH5BAEKAAEALAA
            AAAAJAAkAAAINjI8ZoHv8EISRShubKgA7'''
imgplus = tk.PhotoImage(data = iplus)
imgminus = tk.PhotoImage(data = iminus)
imgcheck = tk.PhotoImage(data = icheck)
# Кнопки для модификации уставки
butminus = tk.Button(frame, image = imgminus, command = setminus)
butminus.place(x = 66, y = yb + 2)
butplus = tk.Button(frame, image = imgplus, command = setplus)
butplus.place(x = 80, y = yb + 2)
butplus = tk.Button(frame, image = imgcheck, command = setcheck)
butplus.place(x = 94, y = yb + 2)

setlab = tk.Label(frame, width = 6, anchor = 'e', textvariable = settxt)
setlab.place(x = 108, y = yb)
yb += hwg

stst = tk.IntVar() # Флаг старт/стопа
stst.set(0) # По умолчанию старт/стоп выключен
ststchb = tk.Checkbutton(frame, text = ' Старт/стоп',
                      command = None, variable = stst, width = 17, anchor = 'w')
ststchb.place(x = 0, y = yb)
yb += hwg
#-------------------------------------------------------------------------------
tk.Frame(frame, width = lw, height = 1, bg = 'grey').place(x = 0, y = yb); yb += 2
#===============================================================================
# Текущая скорость отбора
#===============================================================================
tk.Label(frame, text = 'Отбор, мл/час', width = 13, anchor = 'w').place(x = 5, y = yb)
curqtxt = '%.1f'%(curq)
curqlab = tk.Label(frame, text = curqtxt, width = 6, anchor = 'e')
curqlab.place(x = 108, y = yb)
yb += hwg
#-------------------------------------------------------------------------------
tk.Frame(frame, width = lw, height = 1, bg = 'grey').place(x = 0, y = yb); yb += 2
#===============================================================================
# Давления
#===============================================================================
tk.Label(frame, text = 'Давление,мм.рт.ст.:', width = 19, anchor = 'w').place(x = 5, y = yb)
yb += hwg

tk.Label(frame, text = 'Атмосфера', width = 13, anchor = 'w').place(x = 5, y = yb)
P = '%.1f'%(float(r.lindex(akey, -1)))
atmlab = tk.Label(frame, text = P, width = 7, anchor = 'e')
atmlab.place(x = 100, y = yb)
yb += hwg

ikeys = r.keys('ix*')
ppkey = None
for ikey in ikeys: # Пробегаем по списку всех устройств
  if int(r.hget(ikey, 'fc')) == 0xA7: # Датчик диф.давления
    ppkey = ikey[1:] # Убираем буковку 'i' в начале
if not ppkey is None: # Датчик диф. давления в системе есть
  tk.Label(frame, text = 'Куб', width = 13, anchor = 'w').place(x = 5, y = yb)
  P = '%.1f'%(float(r.lindex(ppkey, -1)))
  potlab = tk.Label(frame, text = P, width = 7, anchor = 'e')
  potlab.place(x = 100, y = yb)
  yb += hwg
#-------------------------------------------------------------------------------
tk.Frame(frame, width = lw, height = 1, bg = 'grey').place(x = 0, y = yb); yb += 2
#===============================================================================
# RMS напряжения питающей сети
#===============================================================================
tk.Label(frame, text = 'Сеть, В', width = 9, anchor = 'w').place(x = 5, y = yb)
V = '%.1f'%(float(r.lindex(rmskey, -1)))
rmslab = tk.Label(frame, text = V, width = 6, anchor = 'e')
rmslab.place(x = 108, y = yb)
yb += hwg
#-------------------------------------------------------------------------------
#tk.Frame(frame, width = lw, height = 1, bg = 'grey').place(x = 0, y = yb); yb += 2
#===============================================================================
# Кнопка для отладочных тестов
#===============================================================================
#def testf():
#  pass
#  global mode
#  mode.set(1)
#  rbcom()
#tk.Button(frame, text = 'Test', command = testf).place(x = 54, y = yb)
#yb += hwg + 8
#===============================================================================
# Графики температур и невязки
#===============================================================================
t = [] # Временные отсчеты
ys = [] # Список списков отсчетов температур
#t = [] # Список отсчетов времени
t = range(1 - lt, 1) # Приблизительное соответствие секундным отсчетам

def drawgraph(canvas): # Поостроение графиков (работает в дочернем потоке)
  global t, ys, tset

  while not r.get(b'lock') is None: time.sleep(0.1) # Ждем конца блокировки БД
  # Для атомарности следующих операций блокируем БД, но не более, чем на 10 сек
  r.set(b'lock', 1); r.expire(b'lock', 10)

  lh = r.llen('time') # "Длина" предыстории на данный момент
  if lh >= lt: lw = lt # Достаточная история для построения графиков
  else: lw = lh # Недостаточно данных для полного графика
  ys.clear()
  
#  if lh < lt: # Не хватает данных для полной оси времени
#    t = range(1 - lt, 1) # Приблизительное соответствие секундным отсчетам
#  else:
#    t = r.lrange(b'time', -lw, -1) # Реальные отсчеты времени
  tcindex = -1
  for i, key in enumerate(tskeys): # Пробегаем по хэшам устройства
    y = r.lrange(key, -lw, -1) # Список последних отсчетов датчика с ключем key
    if lh < lt: # "Укороченный график" - дополним
      y = [0.0]*(lt - lh) + y
    ys.append(y) # Добавим в список отсчетов температур
    if key == tckey: # Это - температура в колонне
      tcindex = i
  r.delete(b'lock') # Разблокируем БД

  ax0.clear() # Настраиваем и рисуем основные графики температру
  ax0.tick_params(labelsize = 'small', labelbottom = False)
  ax0.grid(True, linestyle = '--', lw = 0.5)
  ax0.set_ylabel("Температура, °C")
  ax0.set_ylim(0, 100)

  tcdiff = None
  for ci in range(len(ys)): # 0.13 - 0.15 сек
    ya = np.array(ys[ci]).astype(np.float) # Преобразуем в массив отсчетов температур
    ax0.plot(t, ya, cc[ci], lw = 0.5) # Добавляем график
    if ci == tcindex:
      tcdiff = ya - tset

  ax1.clear() # Настраиваем и рисуем вспомогательный график (невязка)
  ax1.grid(True, linestyle = '--', lw = 0.5)
  ax1.tick_params(labelsize = 'small')
  ax1.set_xlabel("Время, сек")
  ax1.set_ylabel("Невязка, °C")
  ax1.set_ylim(-2, 2)
  ax1.plot(t, tcdiff, cc[tcindex], lw = 0.5)

#  st = time.time()
  canvas.draw() # 0.6 - 0.7 сек
  newdata = False
#  print(time.time() - st)
#===============================================================================
# Управление процессом 
#===============================================================================
def tick(): # Функция вызывается 1 раз в секунду
  global mode, error, baseq, curq, timest, citimer, thread
  # Измерение и отбражение всех параметров
  timert = timestr(time.time() - timest)
  timerlab.configure(text = timert) # Обновим таймер
  if citimer == imgtimer: citimer = imgempty
  else: citimer = imgtimer
  buttimer.configure(image = citimer)

  for i in range(len(tskeys)):
    T = '%.3f'%(float(r.lindex(tskeys[i], -1)))
    Tlabs[i].configure(text = T)
  P = '%.1f'%(float(r.lindex(akey, -1)))
  atmlab.configure(text = P)
  if not ppkey is None: # Датчик диф. давления в системе есть
    P = '%.1f'%(float(r.lindex(ppkey, -1)))
    potlab.configure(text = P)
  wrk = float(r.lindex(rmskey, -1))
  V = '%.1f'%(wrk)
  rmslab.configure(text = V)
  if not error:
    if wrk < 100: # Низкое напряжение в сети - дезактивируем элементы управления
      cstate(tk.DISABLED) # Дизактивируем элементы управления
    else:
      cstate(tk.NORMAL) # Активируем элементы управления

  # Ну и, наконец, подправим динамический график температур, но не каждый раз
  global crefr, nrefr
  if crefr == 0:
    thread = threading.Thread(target = drawgraph(graph))
    thread.start() # Построение графиков происходит в другом потоке
  crefr += 1 # Модифицируем счетчик тиков
  if crefr == nrefr: # Сбрасываем через nrefr тиков
    crefr = 0

  if not error:
    nm = mode.get() # Номер текущего режима
    tc = float(r.lindex(tckey, -1)) # Текущая температура внизу колонных
    td = float(r.lindex(tdkey, -1)) # Текущая температура дефлегматора
    # Анализ специфичных для режимов условий
    if nm == 1: # Разгон
      if tc >= tcrcr: # Окончание режима разгона
        mode.set(2) # Переход в режим холостого хода
        rbcom()
    elif nm == 2: # Холостой ход
      pass
    elif nm == 3: # Отбор голов
      pass # Пока. Потом - выход по объему отобранного спирта
    elif nm == 4: # Отбор подголовников
      pass # Пока. Потом - выход по объему отобранного спирта
    elif nm == 5: # Отбор тела
      pass # Пока. Потом - выход по объему отобранного спирта
    elif nm == 6: # Отбор исправимых хвостов
      if stst.get() > 0: # Выключим режим старт/стоп
        stst.set(0)
      if td >= tdcr: # Окончание отбора хвостов по температуре в дефлегматоре
        mode.set(2) # Переход в режим холостого хода
        rbcom()
    # При отборе подголовников и тела скорость отбора может регулироваться в
    # зависимости от температуры в кубе и колонне. Пока реализован простейший
    # вариант старт/стопа по температуре в нижней части колонны с гистерезисом
    # +-1 квант датчика DS18B20
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    if (nm == 4 or nm == 5) and stst.get() > 0:
      # Простейший старт/стоп с гистерезисом плюс/минус квант от уставки
      if tc >= tset + dt:
        curq = 0.0
      elif tc <= tset - dt:
        curq = baseq
      r.lset(vkey, -1, curq); r.rpush(b'must', vkey)
    curqlab.configure(text = '%.1f'%(curq))
    #~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # Контроль температура воды на выходе дефлегматора (кроме пропарки колонны)
    if nm != 7 and float(r.lindex(twkey, -1)) > twcr:
      print('Температура воды на выходе дефлегматора выше критической')
      mode.set(0); rbcom() # Переходим в режим мониторинга
      vchb.set(0); fchb() # Выключаем контактор
      error = True
      cstate(tk.DISABLED) # Дизактивируем элементы управления
    # Если контроль темп-ры в ТСА включен и она выше критической - вырубаем все
    if ttcontr.get() > 0 and float(r.lindex(ttkey, -1)) >= ttcr:
      print('Температура в ТСА выше критической')
      mode.set(0); rbcom() # Переходим в режим мониторинга
      vchb.set(0); fchb() # Выключаем контактор
      error = True
      cstate(tk.DISABLED) # Дизактивируем элементы управления
#  else:
#    pass
#    if (float(r.lindex(twkey, -1)) <= twcr) or (ttcontr.get() > 0 and \
#        float(r.lindex(ttkey, -1))) < ttcr:
#      error = False
#      cstate(tk.NORMAL) # Активируем элементы управления

  root.after(1000, tick) # Через 1 сек (1000 мс) опять вызовем func
#===============================================================================
# Вспомогательные функции
#===============================================================================
def cstate(state): # Активация/дезактивация элементов управления
  cchb.configure(state = state) # Чекбокс контактора
  for rb in rbms: # Все радиокнопки режимов
    rb.configure(state = state)
#-------------------------------------------------------------------------------
def wclose(): # Метод вызывается перед закрытием главного окна приложения
  global thread
  r.lset(vkey, -1, 0); r.rpush(b'must', vkey) # Выключаем отбор
  r.lset(tkey, -1, 0); r.rpush(b'must', tkey) # Выключаем нагрев
  r.lset(ckey, -1, 0); r.rpush(b'must', ckey) # Выключаем контактор
  if thread.is_alive(): # Подождем завершения потока отрисовки графиков
    thread.join()
  root.destroy() # Закроем главное окно
#===============================================================================
# Главное окно и графическая панель
#===============================================================================
root.geometry('800x%d'%(yb)) # Размер и положение
root.wm_title('lite-cli 0.0.1.4') # Заголовок
root.resizable(width = False, height = False) # Фиксированные размеры окна
root.protocol('WM_DELETE_WINDOW', wclose) # В wclose - обработка закрытия окна

fig = Figure(figsize = (10.0, 6.0))
gs = GridSpec(3, 1)
ax0 = fig.add_subplot(gs[:2, 0]) # Основной график температур
ax1 = fig.add_subplot(gs[2:, 0]) # Вспомогательный график невязки
graph = FigureCanvasTkAgg(fig, master = root)
graph.get_tk_widget().pack(fill = tk.BOTH)

root.after(1000, tick) # Вызываем "тикалку" первый раз 
root.mainloop()
#-------------------------------------------------------------------------------
# История изменений:
# 12.02.2019 - Первая "выкатка" всей системы :) 
#              Разгон, выход на режим холостого хода, выключение - OK
# 13.02.2019
#       1. Добавлен таймер
#       2. Добавлен вывод текущей скорости отбора
#       3. Добавлен и испытан простейший вариант старт/стопа
# 14.02.2019
#       1. Построение графика вынесено в отдельный поток
# 15.02.2019-16.02.2019; v. 0.0.1.3
#       1. Проведен первый полный цикл ректификации, в связи с чем внесены
#          мелкие некритические правки
# 16.02.2019; v. 0.0.1.4
#       1. На канву добавлен второй график (невязка, с масштабом +-2°C)
#       2. Исправлено некорректное завершение потока отрисовки графиков
#       3. Косметические правки и комментарии
# 18.02.2019
#       1. Добавлен режим "Слив" - полностью открытый клапан отбора. Нагрев
#          выключен
# 08.08.2019
#       1. Совместимость с версией Python 3.7.3
#-------------------------------------------------------------------------------
