#-------------------------------------------------------------------------------
# Демон для синхронизации БД Redis и "железа" варианта LITE
# v.0.1.0.0
#-------------------------------------------------------------------------------
# История изменений - в конце файла
# OldBean, 31.01.2019, 08.08.2019
#-------------------------------------------------------------------------------
import time
import math
import sys, os, io, select, tty, fcntl, time, termios
import redis
import threading

# Если данный параметр 1, то система будет пытаться работать с датчиком BMP280,
# при любом другом значении - BMP180
ENABLE_BMP280 = 0

#-------------------------------------------------------------------------------
codes = { 0xA1:'Датчик RMS',
          0xA2:'Твердотельное реле',
          0xA3:'PDM-контроллер',
          0xA4:'PWM-контроллер',
          0xA5:'Хаб 1-Wire',
          0xA6:'1-Wire 10 bit ADC',
          0xA7:'Датчик давления MPX5010DP',
          0xA8:'Датчик расхода',
          0x06:'Отладочный макет',
          0x28:'Датчик температуры DS18B20',
          0xff:'Нормально разомкнутый контакт или пусто',
          0x00:'Нормально замкнутый контакт',
          0x77:'Датчик давления BMP180/BMP280'}
#-------------------------------------------------------------------------------
units = ['', 'В', 'Вт', 'мл/час', '°C', 'мм.рт.ст.', 'мл']
#-------------------------------------------------------------------------------
I2C_SLAVE = 0x703
EE_SIZE = 6
#-------------------------------------------------------------------------------
""" Базовый класс - модуль на шине i2c """
class I2C(object):
  """ Конструктор. addr - адрес на шине i2c """
  def __init__(self, addr):
    self._chFlag = False
    self._addr = addr
    self._rf = io.open("/dev/i2c-1", "rb", buffering = 0)
    self._wf = io.open("/dev/i2c-1", "wb", buffering = 0)
    fcntl.ioctl(self._rf, I2C_SLAVE, addr)
    fcntl.ioctl(self._wf, I2C_SLAVE, addr)
    self._wf.write(b'\x33') # Чтение ROM. В первом байте - код семейства
    time.sleep(0.1)
    self._fc = self._rf.read(1)[0] # Код семейства
    if self._fc == 0xA5:     # Для хаба создадим список кодов семейств
      self._fcs = [0]*8      # датчиков, подключенных к его каналам. Коды
      buf = self._rf.read(9) # содержатся в байтах 1...8 буфера обмена
      for ch in range(8):
        self._fcs[ch] = buf[ch + 1]
      self._coef = [1.0]*8
      self._coefi = [1.0]*8 # 1/coef
      self._offset = [0x00]*8
      self._ui = [0]*8
    else:
      self._coef = 1.0
      self._coefi = 1.0
      self._offset = 0x00
      self._ui = 0
    self.gc()
  """ Деструктор """
  def __del__(self):
    self._rf.close()
    self._wf.close()
  """ Запись калибровочных данных в EEPROM устройства """
  def sc(self, coef = 1.0, offset = 0x00, ui = 0, ch = None):
    if self._fc == 0xA5 and ch is None:
      print('Операция отклонена - для хаба не задан номер канала датчика')
      return
    ip = int(coef) # Целая часть
    fp = int((coef - ip)*10000) # Нормированная дробная часть
    buf = bytearray()
    buf.append(0x4e) # write scratch - записать данные в рабочую область
    if self._fc == 0xA5: # Для хаба добавляем в буфер обмена номер канала
      buf.append(ch & 0xff)
    buf.append(ip & 0xff) # Младший байт целой части
    buf.append((ip >> 8) & 0xff) # Старший байт целой части
    buf.append(fp & 0xff) # Младший байт нормированной дробной части
    buf.append((fp >> 8) & 0xff) # Старший байт нормированной дробной части
    buf.append(offset & 0xff) # Смещение
    buf.append(ui) # Индекс единицы измерения
    self._wf.write(buf) # Запись калибровок в рабочую память устройства
    time.sleep(0.1)
    self._wf.write(b'\x48') # copy scratch - записать блок данных в EEPROM
    time.sleep(0.1)
    self.gc()
  """ Чтение калибровочных данных из EEPROM устройства """
  def gc(self):
    self._wf.write(b'\xb8') # recall EEPROM - считать блок данных из EEPROM
    time.sleep(0.1)
    self._wf.write(b'\xbe') # read scratch - считать данные из рабочей области
    time.sleep(0.1)
    if self._fc == 0xA5: # Для хаба читаем данные всех датчиков
      cd = self._rf.read(72)
      index = 2
      for ch in range(8):
        if self._fcs[ch] == 0x28: # Датчики DS18B20
          self._coef[ch] = 0.0625 # Калибровочный коэффициент
          self._coefi[ch] = 1.0/self._coef[ch]
          self._offset[ch] = 0x00 # Смещение
          self._ui[ch] = 4 # Индекс единицы измерения
        else:
          s = cd[index:index + EE_SIZE]
          ip = s[0] # Младший байт целой части
          ip |= s[1] << 8 # Старший байт целой части
          fp = s[2] # Младший байт дробной части
          fp |= (s[3] << 8) # Старший байт дробной части
          self._coef[ch] = ip + fp*0.0001 # Калибровочный коэффициент
          self._coefi[ch] = 1.0/self._coef[ch]
          self._offset[ch] = s[4] # Смещение
          self._ui[ch] = s[5] # Индекс единицы измерения
        index += 9
    else:
      s = self._rf.read(EE_SIZE)
      ip = s[0] # Младший байт целой части
      ip |= s[1] << 8 # Старший байт целой части
      fp = s[2] # Младший байт дробной части
      fp |= (s[3] << 8) # Старший байт дробной части
      self._coef = ip + fp*0.0001 # Калибровочный коэффициент
      self._coefi = 1.0/self._coef
      self._offset = s[4] # Смещение
      self._ui = s[5] # Индекс единицы измерения
#-------------------------------------------------------------------------------
""" Класс датчика RMS """
class RMS(I2C):
  """ Получение кода RMS, усредненного по 5000 измерений """
  def __init__(self, addr):
    I2C.__init__(self, addr)
    self.gc() # Актуализируем калибровочные данные
  def crms(self):
    self._wf.write(b'\x53')
    cs = self._rf.read(4)
    c = cs[0]
    c |= cs[1] << 8
    c |= cs[2] << 16
    c |= cs[3] << 24
    return c
  def code(self):
    return math.sqrt(self.crms()*0.0002)
  def val(self):
    return self.code()*self._coef
#-------------------------------------------------------------------------------
""" Класс твердотельного реле """
class Relay(I2C):
  def __init__(self, addr):
    I2C.__init__(self, addr)
    self._coef = 1.0
    self._offset = 0
    self._ui = 0
  """ Включение реле """
  def on(self):
    self._wf.write(b'\x52\x01')
  """ Выключение реле """
  def off(self):
    self._wf.write(b'\x52\x00')
#-------------------------------------------------------------------------------
""" Класс контроллера PDM """
class PDM(I2C):
  def __init__(self, addr):
    I2C.__init__(self, addr)
    self._pdm = 0
  """ Установка уровня модуляции в % """
  def setpdm(self, pdm):
    buf = bytearray()
    buf.append(0x52) # Запись данных в регистр регулятора
    buf.append(pdm & 0xff)
    self._wf.write(buf)
    self._pdm = pdm
  """ Включение регулятора на 100% """
  def on(self):
    self._wf.write(b'\x52\x64')
    self._pdm = 100
  """ Выключение регулятора """
  def off(self):
    self._wf.write(b'\x52\x00')
    self._pdm = 0
#-------------------------------------------------------------------------------
""" Класс контроллера PWM """
class PWM(I2C):
  """ Установка уровня модуляции в десятых долях % (т.е. от 0 до 1000) """
  def __init__(self, addr):
    I2C.__init__(self, addr)
    self._pwm = 0
    self._q = 0.0
    self._Q = 0.0
    self._t0 = time.time()
#  def __del__(self):
#    I2C.__del__(self)
  def setpwm(self, pwm):
    now = time.time() # Текущее время
    dt = (now - self._t0)*0.000277777777778 # Время работы в часах
    self._Q += self._q*dt # Модифицируем суммарный расход 
    self._t0 = now # Начинаем новый период отбора с новой скоростью
    buf = bytearray()
    buf.append(0x52) # Запись данных в регистр регулятора
    buf.append(pwm & 0xff)
    buf.append((pwm >> 8) & 0xff)
    self._wf.write(buf)
    self._pwm = pwm
    self._q = 0.001*pwm*self._coef
  @property
  def takeoff(self): # Текущая скорость отбора
    return self._q
  @takeoff.setter
  def takeoff(self, value):
    pwm = int(value*self._coefi*1000 + 0.5)
    if pwm > 1000:
      pwm = 1000
    self.setpwm(pwm)
  @property
  def Q(self): # Текущий суммарный отбор
    now = time.time()
    dt = (now - self._t0)*0.000277777777778 # В часах...
    return self._Q + self._q*dt
  """ Сброс суммарного расхода """
  def resetQ(self):
    self._Q = 0.0
    self._t0 = time.time()
  """ Включение регулятора на 100% """
  def on(self):
    self.setpwm(1000)
  """ Выключение регулятора """
  def off(self):
    self.setpwm(0)
#-------------------------------------------------------------------------------
""" Класс моста i2c-8x1-Wire """
class Hub(I2C):
  def __init__(self, addr):
    I2C.__init__(self, addr)
    self._c = [0x0000]*8
    self._v = [0.0]*8
  def update(self):
    self._wf.write(b'\x53') # Защелкивание данных
    time.sleep(0.1)
    buf = self._rf.read(16)
    for ch in range(8):
      if self._fcs[ch] == 0xff:
        continue
      c = buf[ch*2 + 1]
      c <<= 8
      c |= buf[ch*2]
      self._c[ch] = c
      self._v[ch] = (c - self._offset[ch])*self._coef[ch]
  def val(self):
    return self._v
#-------------------------------------------------------------------------------
""" Класс датчика атмосферного давления"""
if ENABLE_BMP280 == 1:
# Библиотека для датчика BMP280 должна устанавливаться из следующего репозитория:
# git clone https://github.com/adafruit/Adafruit_CircuitPython_BMP280.git
# cd Adafruit_CircuitPython_BMP280
# sudo python3 setup.py
  import board
  import busio
  import adafruit_bmp280

  class BMP(object):
    def __init__(self):
      self.bmp = adafruit_bmp280.Adafruit_BMP280_I2C(busio.I2C(board.SCL, board.SDA))
      (self._coef, self._offset, self._ui) = (0.750062, 0, 5)
      self._fc = 0x77
      pass
    def val(self):
      return self.bmp.pressure*self._coef
else:
# Библиотека для датчика BMP180 должна устанавливаться из следующего репозитория:
# git clone https://github.com/adafruit/Adafruit_Python_BMP.git
# cd Adafruit_Python_BMP
# sudo python3 setup.py
  import Adafruit_BMP.BMP085 as BMP085

  class BMP(object):
    def __init__(self):
      self.bmp = BMP085.BMP085()
      (self._coef, self._offset, self._ui) = (0.00750061683, 0, 5)
      self._fc = 0x77
      pass
    def val(self):
      return self.bmp.read_pressure()*self._coef

#-------------------------------------------------------------------------------
""" Класс контроллера ТЭНа"""
class Teh(object):
  def __init__(self, pdm, rms = None):
    self._pdm = pdm
    self._rms = rms
    if rms is None:
      self._coef = self._pdm._coef*0.01
    else:
      self._coef = rms._coef*rms._coef*0.0002/(220*220)*self._pdm._coef*0.01
  @property
  def power(self): # Мощность нагрева
    pdm = self._pdm._pdm # Уровень модуляции контроллера PDM
    if self._rms is None: # Полагаем что в сети 220В
      return  self._coef*pdm
    crms = self._rms.crms() # Сумма 5000 квадратов кодов АЦП
    return self._coef*crms*pdm
  @power.setter
  def power(self, value):
    if self._rms is None:
      pdm = int(value/self._coef + 0.5)
    else:
      crms = self._rms.crms() # Сумма 5000 квадратов кодов АЦП
      if crms > 0:
        pdm = int(value/(self._coef*crms) + 0.5)
      elif value == 0:
        pdm = 0
      else:
        pdm = 0
        print('Слишком низкое напряжение сети (0 В)')
    self._pdm.setpdm(pdm)
  """ Включение регулятора на 100% """
  def on(self):
    self._pdm.on()
  """ Выключение регулятора """
  def off(self):
    self._pdm.off()
#-------------------------------------------------------------------------------
class SUP(object): # Класс всей установки
  def __init__(self, host = 'localhost', port = 6379, db = 0, dbLen = 172800):
    self.devs = {} # Словарь подключенных i2c-устройств. Ключ - адрес на шине
    cntr = 0
    cur = ['-', '\\', '|', '/']
    self._keyRMS = None
    self._keyTeh = None
    self._cWmax = 0.0 # Коэффициент для оценки максимальной мощности нагрева
    for addr in range(0x03, 0x78): # Пробегаем по адресам шины I2C
      print('\r%s\r'%(cur[cntr%4]), end = '')
      cntr += 1
      try:
        if addr == 0x77: # Датчик атмосферного давления
          key = b'x%2X'%(addr)
          bmp = BMP()
          self.devs[key] = bmp
        else:
          i2c = I2C(addr) # Создаем временный объект. Если все нормально -
          # анализируем код семейства и создаем объект класса-обертки для
          # соответствующего устройства
          key = b'x%2X'%(addr)
          if i2c._fc == 0xA1: # Создадим датчик RMS
            self.devs[key] = RMS(addr)
            if self._keyRMS is None: # Первый датчик RMS и будет тем датчиком
              self._keyRMS = key     # при помощи которого регулируем мощность
          elif i2c._fc == 0xA2: # Создадим контроллер контактора
            self.devs[key] = Relay(addr)
          elif i2c._fc == 0xA3: # Создадим контроллер PDM-модулятора
            self.devs[key] = PDM(addr)
            if self._keyTeh is None: # Предполагаем, что первый (на шине)
              self._keyTeh = key     # контроллер PDM управляет нагревом ТЭНа
          elif i2c._fc == 0xA4: # Создадим контроллер PWM-модулятора
            self.devs[key] = PWM(addr)
          elif i2c._fc == 0xA5: # Это хаб - создадим соотвествующий объект
            self.devs[key] = Hub(addr)
      except IOError: # На этом адресе ничего нет - просто игнорируем
        pass
      except BaseException as error: # О других исключениях будем говорить ;)
        print('\n', str(error))
    # Пока будем строги - и датчик RMS, и контроллер ТЭНа должны быть в системе
    if self._keyRMS is None or self._keyTeh is None:
      print('Не найден датчик RMS и/или PDM-контроллер ТЭНа')
      print('Завершение работы')
      sys.exit(1)
    urms = self.devs[self._keyRMS].val() # Напряжение понадобится еще и ниже!!!
#    if urms < 100.0:
#      print('Слишком низкое напряжение сети (%.1f В)'%(urms))
#      print('Завершение работы')
#      sys.exit(1)
    # Создаем объет "ТЭН". Это "симбиоз" PDM-контроллера и датчика RMS
    self.teh = Teh(self.devs[self._keyTeh], self.devs[self._keyRMS])
    # Теперь начинаем работать с редиской
    self.r = redis.StrictRedis(host, port, db) # Соединяемся с БД
    self.dbLen = dbLen # Размер циклической БД (по умолчанию история - 2 суток)
    self.rrdFlag = False # Циклическая БД (Round-robin Database - RRD)
    self.r.flushdb() # Полностью очищаем базу данных
    self.r.set(b'lock', b'1') # Блокируем внешний доступ к БД
    for key in codes.keys(): # Коды типов устройств
      self.r.hset(b'codes', int(key), codes[key])
    for i in range(len(units)): # Список единиц измерения
      self.r.rpush(b'units', units[i])
    for key in sorted(self.devs.keys()): # Пробегаем по всем устройствам
      ikey = b'i' + key # Ключ для хэша с информацией об устройстве
      dev = self.devs[key]
      if dev._fc == 0xA1: # Датчик RMS
        v = dev.val() # Получим текущее (первое) значение датчика RMS
        self.r.rpush(key, v) # Поместим его в список истории
        self.ctodb(dev, ikey) # Заполним хэш калибровочной информацией
      # Для остальных устройств делаем то же самое
      elif dev._fc == 0xA2: # Твердотельное реле
        self.r.rpush(key, 0) # Начальное состояние - выключено. Это - в БД
        self.r.rpush(b'must', key) # Состояние контроллера нужно обновить
        self.ctodb(dev, ikey) # Заполним хэш калибровочной информацией
      elif dev._fc == 0xA3: # PDM-контроллер
        if key == self._keyTeh: # Это контроллер ТЭНа
          self.r.rpush(key, 0.0) # Должен быть выключен
          self._cWmax = dev._coef/48400.0
          # Для ТЭНа есть дополнительное поле 'max' - максимальная мощность
          # нагрева при текущем напряжении (она же может изменяться)
          self.r.hset(ikey, b'max', self._cWmax*urms*urms)
        else: # Это PDM-контроллер, не связанный с ТЭНом
          self.r.rpush(key, 0)
        self.ctodb(dev, ikey)  # Заполним хэш калибровочной информацией
        self.r.rpush(b'must', key) # Состояние контроллера нужно обновить
      elif dev._fc == 0xA4: # PWM-контроллер (клапан отбора)
        self.r.rpush(key, 0.0)
        self.r.rpush(b'must', key) # Состояние контроллера нужно обновить
        self.ctodb(dev, ikey)  # Заполним хэш калибровочной информацией
        self.r.hset(ikey, b'max', dev._coef) # Для них максимум фиксирован
      elif dev._fc == 0xA5: # 1-Wire хаб
        dev.update() # Обновим данные с подключенных датчиков
        v = dev.val() # Список показаний датчиков
        for ch in range(8): # Пробегаем по всем каналам хаба
          if dev._fcs[ch] == 0xff: # Игнорируем "пустые" каналы. Пока.
            continue
          chkey = key + (b'_%1d'%(ch)) # Формируем ключ для этого датчика в БД
          ichkey = b'i' + chkey        # В начале ключа - 'i', в конце - номер
          self.r.rpush(chkey, v[ch]) # Первое значение в списке истории
          self.ctodb(dev, ichkey, ch) # Заполним хэш калибровочной информацией
      elif dev._fc == 0x77: # Датчик атмосферного давления (BMP)
        v = dev.val()
        self.r.rpush(key, v)
        self.ctodb(dev, ikey) # Заполним хэш калибровочной информацией
    self.tst = time.time()
    self.r.set(b'tst', self.tst) # Время начала мониторинга всех параметров
    self.r.rpush(b'time', b'%d'%(time.time() - self.tst)) # Относительное время
    self.r.set(b'status', b'r') # Статус БД для внешнего управления b'r' - работа,
                                # b'p' - пауза, b'q' - завершение работы
    self.r.delete(b'lock') # Разблокируем внешний доступ к БД

  def info(self): # Полная информация о подключенном оборудовании
    s = 'Найдено следующее оборудование:\n'
    for key in sorted(self.devs.keys()): # Пробегаем по сорт. списку устройств
      dev = self.devs[key] # Ссылка на соотвествующий объект класса-обертки
      s += key.decode() + ' ' + codes[dev._fc]
      if dev._fc in (0xA1, 0xA3, 0xA4): # Добавим калибровки
        s += ' (% 4.4f, %d, %d)\n'%(dev._coef, dev._offset, dev._ui)
      elif dev._fc == 0xA5: # Смотрим что подключено к хабу 1-Wire
        s += '. К каналам хаба подключены:\n'
        for ch in range(8):
          fcc = dev._fcs[ch]
          s += '    %d %s'%(ch, codes[fcc])
          if fcc in (0x00, 0xA7):
            s += ' (%4.4f, %d, %d)\n'%(dev._coef[ch], dev._offset[ch], dev._ui[ch])
          else:
            s += '\n'
      else:
        s += '\n'
    return s

  def rpushlpop(self, key, val, flag): # Вставка в кольцевой список
    self.r.rpush(key, val) # Вставляем в конец списка
    if flag: # А для кольцевого списка удаляем первый элемент
      self.r.lpop(key)

  def ctodb(self, dev, ikey, ch = None): # Копирование калибровки в БД
    # Для устройств, подключенных к хабам, необходимо задавать номер канала ch
    if dev._fc == 0x77: # Датчик давления BMP
      self.r.hset(ikey, b'fc', dev._fc)
      self.r.hset(ikey, b'coef', 0.00750061683)
      self.r.hset(ikey, b'offset', 0)
      self.r.hset(ikey, b'ui', 5)
    elif dev._fc == 0xA5: # Хаб
      self.r.hset(ikey, b'fc', dev._fcs[ch])
      self.r.hset(ikey, b'coef', dev._coef[ch])
      self.r.hset(ikey, b'offset', dev._offset[ch])
      self.r.hset(ikey, b'ui', dev._ui[ch])
    else: # Остальные устройства
      self.r.hset(ikey, b'fc', dev._fc)
      self.r.hset(ikey, b'coef', dev._coef)
      self.r.hset(ikey, b'offset', dev._offset)
      self.r.hset(ikey, b'ui', dev._ui)

  def ctodev(self, ikey, dev, ch = None): # Запись калибровки в устройства
    c = self.r.hgetall(ikey)
    if dev._fc == 0x77: # Датчик давления BMP - игнорируем
      pass
    elif dev._fc == 0xA5: # Хаб
      dev.sc(float(c[b'coef']), int(c[b'offset']), int(c[b'ui']), ch)
    else:
      dev.sc(float(c[b'coef']), int(c[b'offset']), int(c[b'ui']))

  def sync(self): # Синхронизация "железа" и БД (в обе строны)
    while not self.r.get(b'lock') is None: time.sleep(0.1) # Ждем разблокирования БД если есть
    self.r.set(b'lock', b'1') # Блокируем БД для атомарности всей синхронизации
    # Ключи устройств, у которых необходимо обновить калибровку, находятся
    # в списке БД с ключем b'mucal'
    while self.r.llen(b'mucal') > 0:
      key = self.r.lpop(b'mucal')
      dev = self.devs[key[:3]]
      ikey = b'i' + key
      if dev._fc == 0xA5: # Это - хаб
        ch = int(key[4:]) # Выделим из имени ключа номер канала
        sup.ctodev(ikey, dev, ch)
      else:
        sup.ctodev(ikey, dev)
    # Синхронизируем состояние исполнительных устройст. Для контроллера ТЭНа
    # это делаем каждый раз, а для остальных - только если они есть в списке БД
    # с ключем b'must'
    self.teh.power = float(self.r.lindex(self._keyTeh, -1))
    while self.r.llen(b'must') > 0: # Пошли по остальным исполнительным устройствам
      key = self.r.lpop(b'must')
      if key == self._keyTeh:
        continue
      dev = self.devs[key] # Ссылка на объект-устройство
      val = self.r.lindex(key, -1) # Текущее значение величины, которое нужно
                                   # послать устройству
      if dev._fc == 0xA2: # Твердотельное реле 
        if int(val) == 0: dev.off() # Выключаем реле
        else: dev.on() # Включаем реле по любому ненулевому значению
      elif dev._fc == 0xA3: # PDM-контроллер,
        if key != self._keyTeh: # но не контроллер ТЭНа!!!
          dev.setpdm(int(val))
      elif dev._fc == 0xA4: # PWM-контроллер (клапан отбора)
        dev.takeoff = float(val)
    # Теперь будем обновлять БД
    if not self.rrdFlag: # Может быть пора уже "закольцевать" БД ?
      if self.r.llen('time') >= self.dbLen:
        self.rrdFlag = True # Установим флаг кольцевой БД

    for key in sorted(self.devs.keys()): # Пробегаем по устройствам
      dev = self.devs[key]
      if dev._fc == 0xA1: # Датчик RMS
        urms = dev.val()
        self.rpushlpop(key, urms, self.rrdFlag)
        # Если этот датчик используется для контроля мощности нагрева ТЭНа, то
        # обновим поле, где хранится максимально доступная мощность нагрева
        if key == self._keyRMS:
          self.r.hset(b'i' + self._keyTeh, b'max', self._cWmax*urms*urms)
      # Для исполнительных устройств - просто копируем последнее значение
      elif dev._fc == 0xA2: # Твердотельное реле
        self.rpushlpop(key, self.r.lindex(key, -1), self.rrdFlag)
      elif dev._fc == 0xA3: # PDM-контроллер
        self.rpushlpop(key, self.r.lindex(key, -1), self.rrdFlag)
      elif dev._fc == 0xA4: # PWM-контроллер
        self.rpushlpop(key, self.r.lindex(key, -1), self.rrdFlag)
      elif dev._fc == 0xA5: # 1-Wire хаб
        dev.update()  # Обновим в хабе данные с датчиков
        v = dev.val() # Список показаний датчиков
        for ch in range(8): # Пробегаем по каналам хаба
          if dev._fcs[ch] == 0xff: # Игнорируем "пустые" каналы. Пока.
            continue
          chkey = key + (b'_%1d'%(ch))
          self.r.rpush(chkey, v[ch])
      elif dev._fc == 0x77: # Датчик атмосферного давления (BMP)
        self.rpushlpop(key, dev.val(), self.rrdFlag)
    self.r.rpush(b'time', b'%d'%(time.time() - self.tst)) # Относительное время
    self.r.delete(b'lock') # Снимаем блокировку БД
#-------------------------------------------------------------------------------
if __name__ == '__main__':
  sup = SUP() # Создаем и конфигурируем класс установки
  print(sup.info())
  time.sleep(1)

  pStatus = sup.r.get('status') # Предыдущее состояние системы
  run = True # Флаг штатной работы системы
  while True:
    st = time.time()
    if run:
      print('\r%d\r'%((time.time() - float(sup.r.get(b'tst')))), end = '')
    status = sup.r.get('status')
    if status != pStatus: # Статус был изменен клиентом - реагируем
      if status in (b'q', b'Q', b'\xd0\x99', b'\xd0\xb9'): break
      elif status in (b'p', b'P', b'\xd0\x97', b'\xd0\xb7'): # Пауза
        run = False
        print('\rпауза...\r', end = '')
      else: # Работа
        run = True
        print('\r          \r', end = '')
      pStatus = status
    if run:
      sup.sync()
    while int(time.time() - st) == 0: pass # Ждем до 1 секунды
  print('\n')
#-------------------------------------------------------------------------------
# 30.01.2019
#   1. Отлажен базовый функционал синхронизатора
# 31.01.2019
#   1. Добавлена возможности установки калибровок модулей в БД с последующей
#      симнхронизацией с EEPROM соотвествующих модулей
# 01.02.2019
#   1. В БД добавлены коды типов устройств (коды семейств) и единиц измерения
# 03.02.2019
#   1. Добавлена возможность timeout-а для внешних блокировок БД со стороны
#      клиентов (задавать expire для ключа b'lock' при установке блокировки).
#   2. Полностью переписана функция sync класс SUP
# 08.08.2019
#   1. Совместимость с версией Python 3.7.3
#   2. Добавлена возможность выбора типа датчика давления BMP280 или BMP180
#-------------------------------------------------------------------------------

